import numpy as np
import loopy as lp
import pyopencl as cl
from loopy.kernel.tools import _add_and_infer_dtypes_overdetermined
from loopy.compiled import CompiledKernel
from loopy.auto_test import make_ref_args, make_args
import time

ctx = cl.create_some_context()
queue = cl.CommandQueue(
    ctx, properties=cl.command_queue_properties.PROFILING_ENABLE)

param_dict = dict(n=512, m=512, l=512)
dtype = np.float32

# group size
by = 16
bx_list = [16, 24]

# define kernel
knl = lp.make_kernel(
        "{[i,k,j]: 0<=i<n and 0<=k<m and 0<=j<l}",
        [
            "c[i, j] = sum(k, a[i, k]*b[k, j])"
        ], name="nonsq_mm", assumptions="n,m,l >= 1")


knl = _add_and_infer_dtypes_overdetermined(knl,
                                           dict(a=dtype, b=dtype, c=dtype))

# create arrays with random numbers
cl_kernel_info = CompiledKernel(ctx, knl).cl_kernel_info(frozenset())
args, arg_data = make_ref_args(knl, cl_kernel_info.implemented_data_info,
                               queue, param_dict)
arg_arrays = make_args(knl, cl_kernel_info.implemented_data_info, queue,
                       arg_data, param_dict)

ref_knl = knl

# loop over bx values
for bx in bx_list:

    knl = ref_knl

    # transform
    knl = lp.split_iname(knl, "i", by, outer_tag="g.0", inner_tag="l.1")
    knl = lp.split_iname(knl, "j", bx, outer_tag="g.1", inner_tag="l.0")
    knl = lp.split_iname(knl, "k", by)
    knl = lp.extract_subst(knl, "a_acc", "a[i1,i2]", parameters="i1, i2")
    knl = lp.extract_subst(knl, "b_acc", "b[i1,i2]", parameters="i1, i2")
    knl = lp.precompute(knl, "a_acc", "k_inner,i_inner")
    knl = lp.precompute(knl, "b_acc", "j_inner,k_inner")

    knl = lp.set_options(knl, write_cl=True)
    compiled = CompiledKernel(ctx, knl)

    # run
    print("running   bx:", bx, "by:", by)
    times = []
    for t in range(20):
        queue.finish()
        tstart = time.time()
        evt, out = compiled(queue, **arg_arrays)
        queue.finish()
        tfinish = time.time()
        times.append(tfinish-tstart)
    print("time: ", np.average(times[2:]))
    print("="*80)
